@@ -164,17 +164,12 @@ function url_print_header($url, $cm, $course) {
  * @param object $url
  * @param object $cm
  * @param object $course
- * @param bool $ignoresettings print even if not specified in modedit
+ * @param bool $notused This variable is no longer used.
  * @return void
  */
-function url_print_heading($url, $cm, $course, $ignoresettings=false) {
+function url_print_heading($url, $cm, $course, $notused = false) {
     global $OUTPUT;
-
-    $options = empty($url->displayoptions) ? array() : unserialize($url->displayoptions);
-
-    if ($ignoresettings or !empty($options['printheading'])) {
-        echo $OUTPUT->heading(format_string($url->name), 2, 'main', 'urlheading');
-    }
+    echo $OUTPUT->heading(format_string($url->name), 2);
 }
 
 /**
